use strict;
use warnings FATAL => 'all';
use Test::More tests => 16;
use DE_EPAGES::Amazon::API::Tools;
use DE_EPAGES::Core::API::Script qw( RunScript );



sub _TestSplitFullname{
    my ($Fullname, $FirstnameExpected, $LastnameExpected) = @_;

    my ( $Firstname, $Lastname ) = DE_EPAGES::Amazon::API::Tools::SplitFullname( $Fullname );

    $Fullname //= '(undef)';
    is($Firstname, $FirstnameExpected, "firstname=[$Firstname] from [$Fullname]");
    is($Lastname, $LastnameExpected, "lastname=[$Lastname] from [$Fullname]");

}

sub Main {
  _TestSplitFullname(   'Friedrich Schiller',
                        'Friedrich', 'Schiller');

  _TestSplitFullname(   'Schiller,Friedrich',
                        'Friedrich', 'Schiller');

  _TestSplitFullname(   'Schiller, Friedrich',
                        'Friedrich', 'Schiller');

  _TestSplitFullname(   'Schiller,    Friedrich',
                        'Friedrich', 'Schiller');

  _TestSplitFullname(   'Goethe, Johann Wolfgang',
                        'Johann Wolfgang', 'Goethe');

  _TestSplitFullname(   'Goethe,Johann Wolfgang',
                        'Johann Wolfgang', 'Goethe');

  _TestSplitFullname(   'Goethe',
                        '', 'Goethe');

  _TestSplitFullname(   'Johann-Wolfgang Goethe',
                        'Johann-Wolfgang', 'Goethe');
}

RunScript(Sub=>\&Main, Debug=>1);
