use strict;

use Data::Dumper;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::ServicePackageOptionConstants::MappingOptionBlacklist qw (
    %MAPPINGOPTIONBLACKLIST
    CanCombineOptions
);
use Test::More;

sub Main {
    our %MAPPINGOPTIONBLACKLIST = (
    'PackageType1' => [
        ['Option111','Option112','Option113'],
        ['Option121','Option122','Option123'],
        ['Option131','Option132','Option133'],
    ],
    'PackageType2' => [
        ['Option211','Option212','Option213'],
        ['Option221','Option222','Option223'],
        ['Option231','Option232','Option233'],
    ],
);

    TestCanCombineOptions();
    done_testing();
}

sub TestCanCombineOptions {
    my $True = 1;
    my $False = 0;
    is(CanCombineOptions('PackageType2',['Option221','Option224']),$True, 'can combine in PackageType2 Option221 & Option224');
    is(CanCombineOptions('PackageType1',['Option111','Option114']),$True, 'can combine in PackageType1  Option111 & Option114');

    is(CanCombineOptions('PackageType2',['Option221','Option213']),$True, 'can combine in PackageType2 wich belong to other Gropus');
    is(CanCombineOptions('PackageType2',['Option221','Option233']),$True, 'can combine in PackageType2 wich belong to other Gropus');

    is(CanCombineOptions('PackageType1',['Option211','Option212']),$True, 'can combine in PackageType1 options which where forbidden in Packtype2');

    is(CanCombineOptions('PackageType2',['Option221','Option222']),$False, 'can not combine Option221 & Option224');
    is(CanCombineOptions('PackageType1',['Option111','Option112']),$False, 'can combine Option221 & Option224');

    eval {
        CanCombineOptions('WrongName',['one','two']);
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code,'UnknownPackageType','correct Error handling');
       $Error->code
    }
    eval {
        CanCombineOptions('WrongName',['wrong','array','lenght']);
    };
    if (ExistsError()) {
        my $Error = GetError();
        is($Error->code,'WrongAmountOfOptions','correct Error handling');
       $Error->code
    }
}

RunScript( Sub => \&Main );