use strict;

use Data::Dumper;
use DE_EPAGES::Core::API::Error qw ( Error ExistsError GetError );
use DE_EPAGES::Core::API::Log qw ( LogError );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::ServicePackageOptionConstants::MappingServiceTypePackageType qw (
    %MAPPINGSERVICETYPEPACKAGETYPE
    GetPackageTypes
    GetServiceTypes
);
use Test::More;

our $PACKAGEBLOCK1 = ['PackageType11','PackageType12','PackageType13'];
our $PACKAGEBLOCK2 = ['PackageType21','PackageType22','PackageType23'];
sub Main {
    %MAPPINGSERVICETYPEPACKAGETYPE = (
        'ServiceType1' => $PACKAGEBLOCK1,
        'ServiceType2' => $PACKAGEBLOCK2,
    );
    TestGetServiceTypes();
    TestGetPackageTypes();
    done_testing();
}

sub TestGetServiceTypes {
    is_deeply(GetServiceTypes(), [ 'ServiceType1', 'ServiceType2' ], 'give the Correct Service Types');;
}
sub TestGetPackageTypes {
    is_deeply(GetPackageTypes(), [@$PACKAGEBLOCK1, @$PACKAGEBLOCK2], 'give the Correct Package Types');
    is_deeply(GetPackageTypes('ServiceType1'), $PACKAGEBLOCK1, 'give the Correct Package Types for ServiceType1');
    is_deeply(GetPackageTypes('ServiceType2'), $PACKAGEBLOCK2, 'give the Correct Package Types for ServiceType2');
    eval {
        GetPackageTypes('NotAServiceType');
    };
    if (ExistsError()) {
        my $Error = GetError();
        Error($Error) if ( $Error->code eq 'NativDie' );
        is($Error->code,'NotAValidServiceType','correct Error handling');
        is($Error->{'Vars'}->{'Parameter'},'NotAServiceType','correct Error handling Parameter value');
        LogError($Error);
    }
}

RunScript( Sub => \&Main );