use strict;

use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::DHLIntraship::API::Constants qw ( $DEFAULTSELECTNAME );
use DE_EPAGES::DHLIntraship::API::ServiceBuilder;
use Test::More;

sub Main {
    TestSetValueElement();
    TestCreateServiceKnot();
    RegesstionTestGetServiveKnot();
    done_testing();
}

#----------------------------------------------------------------------------------------
sub TestCreateServiceKnot{
    my $ServiceController = DE_EPAGES::DHLIntraship::API::ServiceBuilder->new('ElementName');
    $ServiceController->createServiceElement();
    is_deeply( $ServiceController->{'ServiceKnot'}, undef, 'createServiceElement - test if there are no parameters - undef' );

    $ServiceController->setServiceElement([$DEFAULTSELECTNAME,'one','two']);
    $ServiceController->createServiceElement();
    my $aExpectedResult = { 'Name'     => 'ElementName',
                            'Options'  => [$DEFAULTSELECTNAME,'one','two'],
                          };
    is_deeply( $ServiceController->{'ServiceKnot'}, $aExpectedResult, 'createServiceElement - test if only two Services are left - expected Hash' );

    $ServiceController->setServiceElement([$DEFAULTSELECTNAME,'one','two'],'SelectedOption');
    $ServiceController->createServiceElement();
    $aExpectedResult = { 'Name'            => 'ElementName',
                         'Options'         => [$DEFAULTSELECTNAME,'one','two'],
                         'SelectedOption'  => 'SelectedOption',
                       };
    is_deeply( $ServiceController->{'ServiceKnot'}, $aExpectedResult, 'createServiceElement - test if only two Services are left and there is a selected Option - expected Hash');

    $ServiceController->setServiceElement([$DEFAULTSELECTNAME,'one']);
    $ServiceController->createServiceElement();
    $aExpectedResult = { 'Name'            => 'ElementName',
                         'Options'         => [$DEFAULTSELECTNAME,'one'],
                       };
    is_deeply( $ServiceController->{'ServiceKnot'}, $aExpectedResult, 'createServiceElement - test if only one Service is left - expected Hash');


    $ServiceController->setServiceElement([$DEFAULTSELECTNAME,'one','two'],'SelectedOption');
    $ServiceController->{'ValueElement'} = 'A Value'; # mocked Value
    $ServiceController->createServiceElement([$DEFAULTSELECTNAME,'one','two'],'SelectedOption');
    $aExpectedResult = { 'Name'            => 'ElementName',
                         'Options'         => [$DEFAULTSELECTNAME,'one','two'],
                         'SelectedOption'  => 'SelectedOption',
                         'Value'           => 'A Value',
                       };
    is_deeply( $ServiceController->{'ServiceKnot'}, $aExpectedResult, 'createServiceElement - test with user Values (like max Amount of) - expected Hash');
}

#----------------------------------------------------------------------------------------
sub TestSetValueElement{
    my $ServiceController = DE_EPAGES::DHLIntraship::API::ServiceBuilder->new();
    my $hAmountInfo= { 'MinOrMax' => 'MAX','MinMaxAmount'=> 123 };
    my $UserAmount = 200;

    $ServiceController->setValueElement( $hAmountInfo, $UserAmount );
    $ServiceController->createValueElement();
    is_deeply( $ServiceController->{'ValueElement'},
               { 'UserAmount' => 200, 'MinOrMax' => 'MAX', 'MinMaxAmount' => 123, },
               'setValueElement - build this a valid Hash with user amount - Hash' );

    $ServiceController->setValueElement( $hAmountInfo, undef );
    $ServiceController->createValueElement( $hAmountInfo, undef );
    is_deeply( $ServiceController->{'ValueElement'},
              { 'MinOrMax' => 'MAX', 'MinMaxAmount' => 123 },
             'setValueElement - build this a valid Hash without user amount - Hash' );

    $ServiceController->setValueElement( 1, undef );
    $ServiceController->createValueElement( 1, undef );
    is_deeply( $ServiceController->{'ValueElement'},
              undef ,
               'setValueElement - build this a valid Hash without user amount and hAmountInfo eq 1 - Hash' );
}

#----------------------------------------------------------------------------------------
sub RegesstionTestGetServiveKnot{

     my $ServiceController = DE_EPAGES::DHLIntraship::API::ServiceBuilder->new('NewName');

     $ServiceController->setServiceElement( [ $DEFAULTSELECTNAME, 'eins', 'zwei' ], 'zwei');
     $ServiceController->setValueElement( {'MinOrMax' => 'MIN', 'MinMaxAmount' => 321 } ,666);
     my $hExpectedResult = {
        'Value' => { 'MinMaxAmount' => 321, 'UserAmount' => 666, 'MinOrMax' => 'MIN' },
        'Options' => [ $DEFAULTSELECTNAME, 'eins', 'zwei' ],
        'Name' => 'NewName',
        'SelectedOption' => 'zwei',
     };
     my $hResult = $ServiceController->getServiceKnot();
     is_deeply($hResult, $hExpectedResult, 'getServiveKnot - regressiontest if the knot have the correct shape - knote as hash');
}
RunScript( Sub => \&Main );
