use Test::More;
use strict;

use DE_EPAGES::Core::API::Script qw( RunScript );
use DE_EPAGES::Object::API::Factory qw( LoadRootObject );
use DE_EPAGES::Database::API::Connection qw( RunOnStore GetTestSite );

sub Main {
    my $System = LoadRootObject();
    my $StyleGroups = $System->get('StyleGroups');

    my $Branches = @{$StyleGroups}[0];
    ok($Branches->alias eq 'Branches', 'Branches should be available as stylegroups.');

    my $Layouts = @{$StyleGroups}[1];
    ok($Layouts->alias eq 'Layouts', 'Layouts should be available as stylegroups.');

    foreach my $Layout ( @{$Layouts->folder('StyleGroups')->children} ) {
        my $LayoutName = $Layout->alias;

        my $StyleGroupMaps = $Layout->get('StyleGroupMaps');
        ok(scalar @$StyleGroupMaps > 0, "Layout $LayoutName should have at least 1 Template.");

        foreach my $StyleGroupMap ( @$StyleGroupMaps ) {
            my $StyleTemplate = $StyleGroupMap->get('StyleTemplate');
            fail('StyleGroupMap should have an associated StyleTemplate.') unless $StyleTemplate;
        }
    }

    foreach my $Branche ( @{$Branches->folder('StyleGroups')->children} ) {
        my $BrancheName = $Branche->alias;
        ok( $Branche->get('DefaultStyleHeaderImage'), "Branche $BrancheName should have a default header image.");

        my $HeaderImages = $Branche->get('StyleHeaderImages');
        ok( scalar @$HeaderImages >= 10, "Branche $BrancheName should have at least 10 header images.");

        my $BodyImages = $Branche->get('StyleBodyImages');
        ok( scalar @$BodyImages >= 10, "Branche $BrancheName should have at least 10 body images.");
    }

    done_testing();
}


RunScript( Debug => 1, Sub => sub {
    RunOnStore( Store => GetTestSite(), Sub => sub {
        Main();
    });
});
