/*
 * @copyright		© Copyright 2006-2012, epages GmbH, All Rights Reserved.
 *
 * @module			de_epages.presentation.ui.uploadImage
 *
 * @revision		$Revision: 1.1.2.8 $
 */

jQuery.plugin('de_epages.presentation.ui.uploadImage', {
  plugin : [
    'jQuery.tmpl',
    'jQuery.dict',
    'jQuery.uid',
    'de_epages.presentation.ui.uploader',
    'ep.ui.tooltip',
    'ep.ui.input',
    'ep.alert',
    'ep.ajax',
    'ep.i18n'
  ],
  tmpl : [
    'de_epages.presentation.ui.uploadimage',
    'de_epages.presentation.ui.uploadimage-tooltip',
    'de_epages.presentation.ui.uploadimage-deletedialog'
  ],
  dict : ['de_epages.presentation.dictionary', 'ep.dict']
}, function ($) {
/*
 * @dictionary		de_epages.presentation.dictionary
 *
 * @translation	{DeleteFile}
 * 							{maxWidth}
 * 							{maxHeight}
 * 							{Width}
 * 							{Height}
 * 							{ContentDeleteFile}
 * 							{File}
 *
 *
 *
 * @dictionary		ep.dict
 *
 * @translation {UploadImage}
 *
*/
  var dict = ep.dict('de_epages.presentation.dictionary'),
      epDict = ep.dict('ep.dict');

  $.widget('de_epages.presentation.ui.UploadImage', {
    // Default Options
    options : {
      imagePath : '',
      // See *_setOption*. If set, *image* fades in.
      // If set to null, *image* fades out and removes itself from the DOM.
      defaultImage : '',
      // Path to a default image to be displayed after an image has been deleted.
      scales : false,
      // do not scale image on server. See *_setOption*.
      // Scale width and height are taken from *uploadData.Width*
      // and *uploadData.Height* respectively, if provided.
      // Manipulate the checkbox by sending a data object.
      // Values: name, checked.
      confirm : 1,
      // show delete confirmation dialog. See *_onDeleteFile*.
      accept : 'image/*',
      // A string of MIME types and/or MIME type groups for the *uploader*.
      title : '',
      subtitle : '',
      // see uploadImage.tmpl.html.
      confirmation : '',
      // If set, this is faded in (and out) at end of *_onUpload*.
      imageInfo : {
        Size : null,
        FileName : null,
        Width : null,
        Height : null
      },
      // Format as in response data object. See *_onUpload*
      // Used to render *tooltip*. See *_setOption*
      uploadData : null,
      // for *uploader* widget's ajax request.
      // If *uploadData.Width* or *uploadData.Height* are set,
      // *scales* is set to true during *_create*.
      uploadButtonText : epDict.translate('{UploadImage}'),
      // if set, used for *uploader* widget's button.
      // See uploader.js.
      deleteData : null,
      // for *_deleteFileOnServer*'s ajax request.
      saveData : null,
      // used in *_onUpload*, if it is provided.
      // Some server actions do not actually save
      // the upload file.
      scaleText : '',
      // if provided, *scales* is set to true during *_create*.
      // if NOT, *scales* stays false and the checkbox and its label are not
      // added to the DOM. See uploadImage.tmpl.html.
      deleteText : '{DeleteFile}',
      // See uploadImage.tmpl.html.
      onUpload : null,
      // Callback function. Is called at the end of *_onUpload*.
      // Gets passed the current *imagePath* and *imageInfo*.
      onDelete : null
      // Callback function. Is called at the end of *_deleteFileOnServer*.
      // No arguments are passed.
    },

    _create : function () {
      var self = this;
      var o = self.options;
      // Set up the DOM using jQuery template.
      var renderedView = $.tmpl('de_epages.presentation.ui.uploadimage', {
        title : o.title,
        subtitle : o.subtitle,
        showDeleteFile : o.deleteData || false,
        toggleScales_uid : $.uid('toggleScales'),
        toggleScales_name : o.scales.name || "toggleScales",
        scaleHeight : o.uploadData && o.uploadData.Height,
        scaleWidth : o.uploadData && o.uploadData.Width,
        scaleText : o.scaleText,
        deleteText : o.deleteText
      });
      // Add DOM elements to self.
      $.extend(self, renderedView.tmplItem('elements'));
      // Hide *confirmation*.
      self.confirmationContainer.hide();
      renderedView.dictParse('de_epages.presentation.dictionary', true).appendTo(self.element);

      // Create *tooltip* (disabled!).
      self.tooltip = ep(self.tooltip).uiTooltip({
        orientation : 'cursor',
        context : self.image,
        disabled : true
      });

      // Set up *checkbox* widget,
      // if *toggleScales* is present in the DOM,
      // otherwise set *toggleScales* to []. (See uploadImage.tmpl.html.)
      self.toggleScales = ep(self.toggleScales).uiInput();

      // The checkbox is checked by default,
      // so let us keep *scales* in check.
      // if a scales data object is commited it's not essential to set o.scales to true
      if (o.scaleText && o.scales === false) {
        o.scales = true;
      }
      // Create *uploader* widget.
      self.uploader = de_epages(self.uploader).presentationUiUploader({
        buttonText : o.uploadButtonText,
        callback : $.proxy(self._onUpload, self)
      });

      // Event handling. (The *uploader* handles itself.)
      self.deleteFile.on('click', function () {
        self._onDeleteFile();
      });
      if (self.toggleScales) {
        if (o.scales.checked != undefined) {
          o.scales = o.scales.checked;
        }
        self.toggleScales.on('click', function() {
          self._setOption('scales', !o.scales);
        })
        .attr('checked', o.scales)
        .on('change', function() {
          $(this).prop('defaultChecked', $(this).prop('checked'));
        });
      }

      // The remaining setup is taking care of by *_setOptions*.
      self._setOptions(o);
    },

    _setOption: function (key, value) {
      var self = this;
      var o = self.options;
      this.Inherited(arguments);
      // Now we have 'o[key] = value'.
      switch (key) {
        case 'title' :
          self.title.html(value);
          break;
        case 'subtitle' :
          if (value) {
            self.subtitle
              .html(value)
              .removeClass('HideElement');
          }
          else {
            self.subtitle.addClass('HideElement');
          }
          break;
        case 'confirmation' :
            self.confirmation.html(value);
          break;
        case 'scaleText' :
          if (self.scaleText) {
          // Otherwise *toggleScales* has no label, no checkbox and all that jazz.
            self.scaleText.html(value).dictParse('de_epages.presentation.dictionary', true);
          }
          break;
        case 'scales' :
          // Otherwise we would call *_setUploaderData* twice during *_create*.
          self._setUploaderData();
          break;
        case 'uploadData' :
          self.uploader.attr('disabled', !value);
          self._setUploaderData();
          break;
        case 'deleteData' :
          self.deleteFile.toggleClass('HideElement', !value);
          break;
        case 'accept' :
          self.uploader.presentationUiUploader('option', 'accept', value);
          break;
        case 'imageInfo' :
          if (value) {
            self.tooltip.uiTooltip('option', 'disabled', false);
            if (value.Size) {
              value.formattedSize = ep.i18n.formatBytes(value.Size,0);
            }
            // Setup tooltip text.
            self.tooltip.html($.tmpl('de_epages.presentation.ui.uploadimage-tooltip', value));
          }
          else {
            self.tooltip.uiTooltip('option', 'disabled', true);
          }
          break;
        case 'imagePath' :
          if (value) {
          // Show/Switch the image.
            self.image.fadeOut(function () {
              self.image
                .attr({
                  'src' : value,
                  'alt' : o.imageInfo ? (o.imageInfo.FileName || '') : value
                });
                // This is why imageInfo should be set before imagePath.
              if (self.imageContainer.find('img').length === 0) {
                self.image
                  .appendTo(self.imageContainer)
                  .fadeIn();
              }
              else {
                self.image.fadeIn();
              }
              if (o.defaultImage === value) {
              // Hide *deleteFile*, when *defaultImage* is displayed.
                self.deleteFile.hide();
              }
              else {
              // Otherwise show *deleteFile*.
                self.deleteFile.show();
              }
            });
          }
          else {
          // Hide the image.
            if (o.defaultImage) {
            // Display *defaultImage*
              self._setOptions({
                'imagePath' : o.defaultImage,
                'imageInfo' : null
              });
            }
            else {
            // Remove *image* from DOM
              self.image.fadeOut(function () {
                self._setOption('imageInfo', null);
                self.image.remove(); // Simply removing the 'src' attribute does not suffice.
                self.deleteFile.hide();
              });
            }
          }
          break;
      }
    },

    // Show *confirmation*.
    _showConfirmation : function () {
      if (this.options.confirmation) {this.confirmationContainer.fadeIn('slow').delay(2000).fadeOut('slow');}
    },

    // *uploader* (widget) callback.
    _onUpload : function (data, statusText) {
      var self = this;
      var o = self.options;
      if (data[0] && data[0].success) {
        var image = data[0].success.uploadedFile;
        var imageInfo = data[0].success.imageInfo && data[0].success.imageInfo;
        if (o.saveData) {
        // If present, we ajax the *saveData* over to the server.
          var _uploadData = self.uploader.presentationUiUploader('option', 'data');
          // The following depends on the server action used.
          o.saveData[ _uploadData.Image || 'Filename' ] = _uploadData.UseMediaGallery ? 'MediaGallery/' + imageInfo.FileName : imageInfo.FileName;
          ep.ajax({
            data : o.saveData,
            type : 'POST',
            dataType : 'json'
          })
          .done(function (newData) {
            if (newData.uploadedFile) {
              image = newData.uploadedFile;
            }
            else if (newData.FileName) {
              image = newData.FileName;
            }
            // Set and show the uploaded *image*.
            self._setOption('imagePath', image);
            if (o.onUpload) {
            // Call callback, if present.
              o.onUpload(o.imagePath, imageInfo);
            }
          });
        }
        else {
          // Set and show the uploaded *image*.
          self._setOption('imagePath', image);
          if (o.onUpload) {
          // Call callback, if present.
            o.onUpload(o.imagePath, imageInfo);
          }
        }
        // Set the *tooltip*'s text.
        self._setOption('imageInfo', imageInfo);
        // Show *confirmation* text.
        self._showConfirmation();
      }
    },

    // *deleteFile* (node) callback.
    _onDeleteFile : function () {
      var self = this;
      var o = self.options;
      if (o.confirm) {
      // Ask for confirmation.
        var deleteDialog = ep.alert({
          title : dict.translate('DeleteFile'),
          closeOnEscape : true,
          content : $.tmpl('de_epages.presentation.ui.uploadimage-deletedialog', o.imageInfo)
                      .dictParse('de_epages.presentation.dictionary', true),
          type : 'confirmation',
          okTitle: epDict.translate('Delete'),
          cancel : function () {},
          ok : $.proxy(self._deleteFileOnServer, self)
        });
      }
      else {
        self._deleteFileOnServer();
      }
    },

    // After confirmation delete the image from the server.
    _deleteFileOnServer : function () {
      var self = this;
      var o = self.options;
      var changeAction = o.uploadData.ChangeAction || null;
      var setDefault = function() {
        self._setOptions({
          'imagePath' : o.defaultImage ? o.defaultImage : null,
          'imageInfo' : null
        });
        if (o.onDelete) {
          o.onDelete();
        }
      };
      if (o.saveData || changeAction == "SetView") {
        ep.ajax({
          data : o.deleteData,
          type : 'POST',
          dataType : 'json'
        })
        .done(function () {
          setDefault();
        });
      }
      // If the file isn't stored at the server.
      else {
        setDefault();
      }
    },

    // Set *uploader*'s data object depending on *scales* and *uploaderData*
    _setUploaderData : function () {
      var self = this;
      var o = self.options;
      var _uploadData = $.extend({}, o.uploadData);
      if (!o.scales) {
        delete _uploadData.Width;
        delete _uploadData.Height;
      }
      self.uploader.presentationUiUploader('option', 'data', _uploadData);
    },

    destroy : function () {
      this.mainContainer.remove();
      this.Inherited(arguments);
    }
  });
})(jQuery);